import numpy as np
from PIL import Image, ImageDraw, ImageFont

from model_server.base.process import rescale

def draw_boxes_on_3d_image(roiset, draw_full_depth=False, **kwargs):
    _, _, chroma, nz = roiset.acc_raw.shape
    font_size = kwargs.get('font_size', 18)
    linewidth = kwargs.get('linewidth', 4)

    annotated = np.zeros(roiset.acc_raw.shape, dtype=roiset.acc_raw.dtype)

    for zi in range(0, nz):
        if draw_full_depth:
            subset = roiset.get_df()
        else:
            subset = roiset.get_df().query(f'zi == {zi}')
        for c in range(0, chroma):
            pilimg = Image.fromarray(roiset.acc_raw.data[:, :, c, zi])
            draw = ImageDraw.Draw(pilimg)
            draw.font = ImageFont.truetype(font="arial.ttf", size=font_size)

            for roi in subset.itertuples('Roi'):
                xm = round((roi.x0 + roi.x1) / 2)
                draw.rectangle([(roi.x0, roi.y0), (roi.x1, roi.y1)], outline='white', width=linewidth)

                if kwargs.get('draw_label') is True:
                    draw.text((xm, roi.y0), f'{roi.label:04d}', fill='white', anchor='mb')
            annotated[:, :, c, zi] = pilimg


    if clip := kwargs.get('rescale_clip'):
        assert clip >= 0.0 and clip <= 1.0
        annotated = rescale(annotated, clip=clip)

    return annotated

def draw_box_on_patch(patch, bbox, linewidth=1):
    assert len(patch.shape) == 2
    ((x0, y0), (x1, y1)) = bbox
    pilimg = Image.fromarray(patch)  # drawing modifies array in-place
    draw = ImageDraw.Draw(pilimg)
    draw.rectangle([(x0, y0), (x1, y1)], outline='white', width=linewidth)
    return np.array(pilimg)

def draw_contours_on_patch(patch, contours, linewidth=1):
    assert len(patch.shape) == 2
    pilimg = Image.fromarray(patch)  # drawing modifies array in-place
    draw = ImageDraw.Draw(pilimg)
    for co in contours:
        draw.line([(p[1], p[0]) for p in co], width=linewidth, joint='curve')
    return np.array(pilimg)