import unittest

from model_server.base.models import DummyInstanceSegmentationModel
from model_server.base.roiset import RoiSetMetaParams, RoiSetExportParams
from model_server.conf.testing import output_path
from model_server.extensions.chaeo.conf.testing import multichannel_zstack, pipeline_params
from model_server.extensions.chaeo.workflows import infer_object_map_from_zstack

from tests.test_roiset import BaseTestRoiSetMonoProducts

class TestRoiSetWorkflow(BaseTestRoiSetMonoProducts, unittest.TestCase):

    def test_object_map_workflow(self):
        pp = pipeline_params
        models = [
            self.pxmodel,
            DummyInstanceSegmentationModel(),
        ]

        models = {
            'pixel_classifier': {
                'model': self.pxmodel,
                'params': {
                    'px_class': 0,
                    'px_prob_threshold': 0.6,
                }
            },
            'object_classifier': {
                'name': 'dummy',
                'model': DummyInstanceSegmentationModel(),
            }
        }

        roi_params = RoiSetMetaParams(**{
            'mask_type': 'boxes',
            'filters': {
                'area': {'min': 1e3, 'max': 1e8}
            },
            'expand_box_by': [128, 2]
        })

        export_params = RoiSetExportParams(**{
            'pixel_probabilities': True,
            'patches_3d': {},
            'annotated_patches_2d': {
                'draw_bounding_box': True,
                'rgb_overlay_channels': [3, None, None],
                'rgb_overlay_weights': [0.2, 1.0, 1.0],
                'pad_to': 512,
            },
            'patches_2d': {
                'draw_bounding_box': False,
                'draw_mask': False,
            },
            'patch_masks': {
                'pad_to': 256,
            },
            'annotated_zstacks': {},
            'object_classes': True,
            'dataframe': True,
        })

        infer_object_map_from_zstack(
            multichannel_zstack['path'],
            output_path / 'roiset' / 'workflow',
            models,
            segmentation_channel=pp['segmentation_channel'],
            patches_channel=pp['patches_channel'],
            export_params=export_params,
            roi_params=roi_params,
        )